MultiSimulation

Struct MultiSimulation 

pub struct MultiSimulation<Info: SimulationInfo> { /* private fields */ }
A simulation with support for multiple states.

Implementations

Source

impl<Info: SimulationInfo> MultiSimulation<Info>

pub fn new<T: Into<Info>>(info: T) -> Self

Creates a new MultiSimulation from the provided info with no states.

pub fn add_simulation(&mut self) -> usize

Adds a new simulation state to the simulation and returns its index.

pub fn add_simulation_from_data( &mut self, data: Info::LoadData, ) -> Result<usize, Info::StateLoadingError>

Adds a new simulation state loaded from the provided data to the simulation and returns its index.

Errors

Returns an error if a valid state cannot be loaded from data.

pub fn remove_simulation(&mut self, index: usize) -> Option<Info::State>

Removes the simulation state at the specified index and returns it.

All states after index shift down by one position, so their indices decrease by one. Returns None if index is out of bounds, leaving the simulation unchanged.

pub fn get(&self, index: usize) -> Option<SimulationBorrowRef<'_, Info>>

Retrieves an immutable borrow of the simulation at the specified index.

pub fn get_mut(&mut self, index: usize) -> Option<SimulationBorrowMut<'_, Info>>

Retrieves a mutable borrow of the simulation at the specified index.

pub fn states(&self) -> &[Info::State]

Get access to all simulation states.

pub fn states_mut(&mut self) -> &mut [Info::State]

Get mutable access to all simulation states.

pub fn iter(&self) -> Iter<'_, Info>

Returns an iterator over the simulation states.

pub fn iter_mut(&mut self) -> IterMut<'_, Info>

Returns a mutable iterator over the simulation states.

pub fn release(self) -> Info

Release the info from the simulation again and destroys all states.