SimulationState

Trait SimulationState 

pub trait SimulationState {
    type AccessData: ?Sized;
    type Event: Copy + Ord;
    type EventContainer<'a>: Iterator<Item = Self::Event>
       where Self: 'a;

    // Required methods
    fn data(&self) -> &Self::AccessData;
    fn callables(&self) -> Self::EventContainer<'_>;
    fn revertables(&self) -> Self::EventContainer<'_>;
    fn callable(&self, event: Self::Event) -> bool;
    fn revertable(&self, event: Self::Event) -> bool;
}
The SimulationState trait provides a read-only interface for inspecting the current state of a simulation.

Required Associated Types

type AccessData: ?Sized

The type used to access the current state data.

type Event: Copy + Ord

The type of events that can be called or reverted in the simulation.

type EventContainer<'a>: Iterator<Item = Self::Event> where Self: 'a

The type of container used to access the available events.

Required Methods

fn data(&self) -> &Self::AccessData

Returns a reference to the data which represents the current state.

fn callables(&self) -> Self::EventContainer<'_>

Returns the events that can be called in the current state.

fn revertables(&self) -> Self::EventContainer<'_>

Returns the events that can be reverted in the current state.

fn callable(&self, event: Self::Event) -> bool

Checks if the provided event can be called in the current state.

fn revertable(&self, event: Self::Event) -> bool

Checks if the provided event can be reverted in the current state.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors